@@ -1,14 +1,22 @@
 import asyncio
-import traceback
+import signal
+import time
 import threading
 import multiprocessing
 
-from multiprocessing.synchronize import Event
 from abc import ABC, abstractmethod
-from ..helpers.get_logger import GetLogger
+from ..helpers.get_logger import LoggerFactory
 
 
-LOGGER = GetLogger().get()
+LOGGER = LoggerFactory().get(__name__)
+
+keyboard_interrupt = False
+
+
+def handle_signal(signum, frame):
+    global keyboard_interrupt
+    print(f"Received signal {signum}, shutting down gracefully...")
+    keyboard_interrupt = True
 
 
 class BaseEngine(ABC):
@@ -29,7 +37,7 @@ class BaseEngine(ABC):
     name: str
     loop_name: str
 
-    def __init__(self, run_in_process: bool = False):
+    def __init__(self, run_in_process: bool = False, catch_interrupt: bool = True):
         """
         Initializes the engine instance by creating a new asyncio event loop
         and setting the thread placeholder to None.
@@ -39,90 +47,77 @@ class BaseEngine(ABC):
                                 instead of a threading.Thread. Defaults to False.
         """
         self.run_in_process = run_in_process
-        self.new_loop = asyncio.new_event_loop()
-        # can be Thread or Process depending on multi_process flag
-        # stop_process_event only used in process mode
+        self.catch_interrupt = catch_interrupt
         self.therad = None
         self.process = None
-        self.stop_process_event = None
+        self.stop_event = None
+        self.shutdowned = None
         self.pipeline_task = None
 
-    async def start(self):
+    def start(self):
         """
         Starts the engine by running the `preprocess()` coroutine, launching a new thread
         to host the event loop, and scheduling the `process()` coroutine within that loop.
         """
         LOGGER.info(f"starting {self.name}....")
         if self.run_in_process:
-            self.stop_process_event = multiprocessing.Event()
+            self.stop_event = multiprocessing.Event()
+            self.shutdowned = multiprocessing.Event()
             self.process = multiprocessing.Process(
-                target=self.start_process_loop, args=(self.stop_process_event,)
+                target=self.start_loop, name=self.name, daemon=True
             )
             self.loop_name = self.process.name
             self.process.start()
         else:
-            self.thread = threading.Thread(target=self.start_thread_loop)
+            self.stop_event = threading.Event()
+            self.shutdowned = threading.Event()
+            self.thread = threading.Thread(
+                target=self.start_loop, name=self.name, daemon=True
+            )
             self.loop_name = self.thread.name
             self.thread.start()
 
-    def start_thread_loop(self):
-        """
-        Runs the event loop in the current thread.
+    def start_loop(self):
+        if self.process is not None:
+            # Register handlers for SIGTERM (docker stop) and SIGINT (Ctrl+C)
+            signal.signal(signal.SIGTERM, handle_signal)
+            signal.signal(signal.SIGINT, handle_signal)
 
-        This method is intended to be run in a new thread, and will set the thread-local
-        event loop and run it until stopped.
-        Logs when the loop is stopped.
-        """
-        asyncio.set_event_loop(self.new_loop)
-        try:
-            self.pipeline_task = self.new_loop.create_task(self.pipeline())
-            self.new_loop.run_forever()
-        except Exception as ex:
-            msg_error = traceback.format_exc()
-            LOGGER.error(f"engine crash {msg_error}")
-        LOGGER.info(
-            f"Event loop of engine {self.name} in thread {self.loop_name} stopped"
-        )
-
-    def start_process_loop(self, stop_process_event: Event):
-        """
-        Runs the event loop in the current thread.
-
-        This method is intended to be run in a new thread, and will set the thread-local
-        event loop and run it until stopped.
-        Logs when the loop is stopped.
-        """
         self.new_loop = asyncio.new_event_loop()
         asyncio.set_event_loop(self.new_loop)
 
-        async def runner():
+        async def runner() -> None:
             # Start main process task
             self.pipeline_task = asyncio.create_task(self.pipeline())
 
             # Wait for stop_process_event signal
-            while not stop_process_event.is_set():
+            while self.stop_event and not self.stop_event.is_set():
                 await asyncio.sleep(0.2)
+                if self.catch_interrupt and keyboard_interrupt:
+                    break
 
             self.pipeline_task.cancel()  # cancel main loop if still running
+            try:
+                await self.pipeline_task
+            except asyncio.CancelledError:
+                LOGGER.critical(f"{self.name}: process pipeline canceled")
             self.new_loop.stop()
 
-        try:
-            self.new_loop.create_task(runner())
-            self.new_loop.run_forever()
-        except Exception:
-            msg_error = traceback.format_exc()
-            LOGGER.error(f"engine crash {msg_error}")
-        finally:
-            LOGGER.info(
-                f"Event loop of engine {self.name} in process {self.loop_name} stopped"
-            )
+        self.new_loop.create_task(runner())
+        self.new_loop.run_forever()
+        LOGGER.info(f"Event loop of engine {self.name} in {self.loop_name} stopped")
 
     async def pipeline(self):
+        LOGGER.info(f"preparing {self.name}....")
         await self.prepare()
         try:
+            LOGGER.info(f"executing {self.name}....")
             await self.execute()
         finally:
+            LOGGER.info(f"postparing {self.name}....")
             await self.postpare()
+            if self.shutdowned:
+                self.shutdowned.set()
 
     @abstractmethod
     async def prepare(self):
@@ -154,36 +149,42 @@ class BaseEngine(ABC):
         """
         pass
 
-    async def stop(self):
+    def stop(self):
         """
         Stops the engine gracefully.
 
         Calls the `postprocess()` coroutine, schedules the event loop to stop,
         and joins the engine thread to ensure a clean shutdown.
         """
-        await self.stop_process() if self.run_in_process else await self.stop_thread()
+        self.stop_process() if self.run_in_process else self.stop_thread()
 
-    async def stop_process(self):
-        """
-        Stops the process gracefully.
-
-        Calls the `postprocess()` coroutine, schedules the event loop to stop,
-        and joins the engine thread to ensure a clean shutdown.
-        """
-        if not self.process or not self.stop_process_event:
+    def stop_process(self):
+        if not self.process or not self.stop_event:
             return
-        # signal process to shut down gracefully by running postprocess()
-        # we cant use self.worker.terminate() cause we are running in child process
-        # we have to use events to send shutdown signal
-        self.stop_process_event.set()
+
+        if self.shutdowned and self.shutdowned.is_set():
+            LOGGER.critical(f"{self.name}: it's already shutdowned")
+            self.process = None
+            self.stop_event = None
+            return
+        self.stop_event.set()
+        while self.shutdowned and not self.shutdowned.is_set():
+            time.sleep(0.5)
+        self.process.terminate()
         self.process.join()
         self.process = None
-        self.stop_process_event = None
+        self.stop_event = None
 
-    async def stop_thread(self):
-        if not self.thread or not self.pipeline_task:
+    def stop_thread(self):
+        if not self.thread or not self.pipeline_task or not self.stop_event:
             return
-        self.pipeline_task.cancel()
-        self.new_loop.call_soon_threadsafe(self.new_loop.stop)
+        if self.shutdowned and self.shutdowned.is_set():
+            LOGGER.critical(f"{self.name}: it's already shutdowned")
+            self.thread = None
+            self.stop_event = None
+            return
+        self.stop_event.set()
+        while self.shutdowned and not self.shutdowned.is_set():
+            time.sleep(0.5)
         self.thread.join()
         self.thread = None
